{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: FindReplDemoMain.pas, released 2005-06-23.

The Original Code is part of the FindReplDemo project, written by
Juergen Rathlev for the SynEdit component suite.
All Rights Reserved.

Contributors to the SynEdit project are listed in the Contributors.txt file.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

Known Issues:

Description:
  Demo program for find and replace dialog

Requires:
  SynEdit
    
-------------------------------------------------------------------------------}
unit FindReplDemoMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SynEditTypes, SynEdit, ExtCtrls, StdCtrls, SynEditMiscClasses,
  SynEditSearch, Menus, ComCtrls, XPMan;

type
  TfrmMain = class(TForm)
    pnlButtons: TPanel;
    SynEdit: TSynEdit;
    btnFind: TButton;
    btnLoadText: TButton;
    OpenDialog: TOpenDialog;
    btnReplace: TButton;
    SynEditSearch: TSynEditSearch;
    btnNext: TButton;
    MainMenu: TMainMenu;
    File1: TMenuItem;
    Load1: TMenuItem;
    N1: TMenuItem;
    itmQuit: TMenuItem;
    ools1: TMenuItem;
    Search1: TMenuItem;
    earchnext1: TMenuItem;
    Replace1: TMenuItem;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure btnLoadTextClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure FindNextText(Sender: TObject);
    procedure ReplaceNextText(Sender: TObject);
    procedure SynEditReplaceText(Sender: TObject; const ASearch,
      AReplace: String; Line, Column: Integer;
      var Action: TSynReplaceAction);
    procedure SynEditStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure btnNextClick(Sender: TObject);
    procedure itmQuitClick(Sender: TObject);
  private
    { Private-Deklarationen }
    IniName,
    LastFile : string;
    function GetClientTop : integer;
    function UpdateFindtext : string;
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses IniFiles, FindReplDlg, ConfirmReplDlg;

{ ------------------------------------------------------------------- }
resourcestring
  rsReplace = 'Replace this occurrence'#13'of "%s"'#13+'by "%s"?';

{ ------------------------------------------------------------------- }
const
  DefOptions = [eoAltSetsColumnMode, eoAutoIndent, eoDragDropEditing,
    eoEnhanceEndKey, eoShowScrollHint, eoScrollPastEol, eoSmartTabs,
    eoTabsToSpaces, eoSmartTabDelete, eoGroupUndo];

  (* INI-Sections *)
  CfGSekt = 'Config';

  (* Ini-Vars *)
  iniLast = 'Lastfile';

procedure TfrmMain.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniName:=ChangeFileExt(Application.ExeName,'.ini');
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    LastFile:=ReadString(CfgSekt,iniLast,'');
    Free;
    end;
  SynEdit.Options:=defOptions;
  end;

procedure TfrmMain.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfgSekt,iniLast,LastFile);
    Free;
    end;
  end;

// relative upper position of free client space
function TfrmMain.GetClientTop : integer;
begin
  Result:=Height-ClientHeight+pnlButtons.Height;
  end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  FindReplDialog.LoadFromIni(IniName);          // load history lists from ini file
  end;

procedure TfrmMain.FormResize(Sender: TObject);
var
  ix,iy : integer;
begin
  if Active then begin
    ix:=Left+ClientWidth;             // right margin of edit window
    iy:=Top+GetClientTop;             // upper margin of edit window
    with FindReplDialog do begin
      Left:=ix-Width-15;              // set dialog window position
      Top:=iy+5;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TfrmMain.btnLoadTextClick(Sender: TObject);
begin
  with OpenDialog do begin
    Filename:='';
    Title:='Select text file';
    if Execute then SynEdit.Lines.LoadFromFile(Filename);
    end;
  end;

procedure TfrmMain.itmQuitClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure TfrmMain.btnFindClick(Sender: TObject);
begin
  with FindReplDialog do begin
    Findtext:=UpdateFindtext;
    OnFind:=FindNextText;
    Execute(false);
    end;
  end;

procedure TfrmMain.btnReplaceClick(Sender: TObject);
begin
  with FindReplDialog do begin
    Findtext:=UpdateFindtext;
    OnFind:=FindNextText;
    OnReplace:=ReplaceNextText;
    Execute (true);
    end;
  end;

procedure TfrmMain.btnNextClick(Sender: TObject);
begin
  FindNextText(self);
  end;

{ ------------------------------------------------------------------- }
function TfrmMain.UpdateFindtext : string;
begin
  with SynEdit do begin
    if SelAvail then Result:=SelText
    else Result:=wordAtCursor;
    end;
  end;

// OnFind routine for find text
procedure TfrmMain.FindNextText(Sender: TObject);
begin
  with FindReplDialog do begin
    if Length(FindText)=0 then SysUtils.Beep
    else begin
      if SynEdit.SearchReplace(FindText,'',Options)=0 then begin
        SysUtils.Beep;
        ShowMessage('"'+FindText+'" nicht gefunden!');
        end;
      end
    end;
  end;

// OnReplace routine for replace text
procedure TfrmMain.ReplaceNextText(Sender: TObject);
begin
  with FindReplDialog do begin
    if Length(FindText)=0 then SysUtils.Beep
    else begin
      if SynEdit.SearchReplace(FindText,ReplaceText,Options)=0 then begin
        SysUtils.Beep;
        ShowMessage('"'+FindText+'" nicht gefunden!');
        end;
      end;
    end;
  end;

procedure TfrmMain.SynEditReplaceText(Sender: TObject; const ASearch,
  AReplace: String; Line, Column: Integer; var Action: TSynReplaceAction);
var
  Pos : TPoint;
begin
  Pos:=SynEdit.RowColumnToPixels(DisplayCoord(Column,Line+1));
  with Pos do begin
    X:=X+Left; Y:=Y+Top+GetClientTop;
    end;
  Action:=ConfirmReplDialog.Execute (Pos,Format(rsReplace,[ASearch,AReplace]));
  end;

procedure TfrmMain.SynEditStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if Changes*[scCaretY,scCaretX]<>[] then begin
    with FindReplDialog do if Visible and not Searching then FindText:=UpdateFindtext;
    end;
  end;

end.
